@@ -34,8 +34,8 @@ def plot1d(model, X: np.ndarray, y: np.ndarray, show: Optional[bool] = True) ->
         raise ValueError("plot1d is only supported for 1D input data.")
 
     _ = plt.figure(figsize=(9, 6))
-    n_grid = 100
-    x = linspace(X[:, 0].min(), X[:, 0].max(), num=n_grid).reshape(-1, 1)
+    num = 100
+    x = linspace(X[:, 0].min(), X[:, 0].max(), num=num).reshape(-1, 1)
     y_pred, y_std = model.predict(x, return_std=True)
 
     plt.plot(x, y_pred, "k", label="Prediction")
@@ -55,30 +55,54 @@ def plot1d(model, X: np.ndarray, y: np.ndarray, show: Optional[bool] = True) ->
         plt.show()
 
 
-def generate_mesh_grid(X: np.ndarray, i: int, j: int, n_grid: int = 100):
+def generate_mesh_grid(
+    X: Optional[np.ndarray] = None, i: int = 0, j: int = 1, num: int = 100, lower: Optional[np.ndarray] = None, upper: Optional[np.ndarray] = None, var_type: Optional[List[str]] = None
+):
     """
-    Generate a mesh grid for two selected dimensions of X, and fill the remaining dimensions with their mean values.
+    Generate a mesh grid for two selected dimensions, filling remaining dimensions with their mean values
+    (if X is given) or the mean of the lower and upper bound (if lower and upper are given).
 
     Args:
-        X (np.ndarray): Input data of shape (n_samples, k).
+        X (np.ndarray, optional): Input data of shape (n_samples, k). Required if lower/upper are not given.
         i (int): Index of the first dimension to vary.
         j (int): Index of the second dimension to vary.
-        n_grid (int): Number of grid points per dimension.
+        num (int): Number of grid points per dimension.
+        lower (np.ndarray, optional): Lower bounds for each dimension (shape (k,)).
+        upper (np.ndarray, optional): Upper bounds for each dimension (shape (k,)).
+        var_type (list of str, optional): List of variable types for each dimension. Can be either "num", "int", or "factor".
 
     Returns:
         X_i (np.ndarray): Meshgrid for the i-th dimension.
         X_j (np.ndarray): Meshgrid for the j-th dimension.
-        grid_points (np.ndarray): Grid points of shape (n_grid*n_grid, k) for prediction.
+        grid_points (np.ndarray): Grid points of shape (num*num, k) for prediction.
     """
-    k = X.shape[1]
-    mean_values = X.mean(axis=0)
+    # Check that exactly one of (X) or (lower and upper) is provided
+    if (X is not None and (lower is not None or upper is not None)) or (X is None and (lower is None or upper is None)):
+        raise ValueError("Provide either X or both lower and upper, but not both or neither.")
+
+    if X is not None:
+        k = X.shape[1]
+        mean_values = X.mean(axis=0)
+        x_i = linspace(X[:, i].min(), X[:, i].max(), num=num)
+        x_j = linspace(X[:, j].min(), X[:, j].max(), num=num)
+    else:
+        k = len(lower)
+        mean_values = (np.array(lower) + np.array(upper)) / 2.0
+        x_i = linspace(lower[i], upper[i], num=num)
+        x_j = linspace(lower[j], upper[j], num=num)
+
+    # Masked rounding (using floor) for integer or factor variables
+    if var_type is not None:
+        # For x_i
+        if hasattr(x_i, "__len__"):
+            mask_i = np.array([var_type[i] != "num"] * len(x_i))
+            x_i = np.where(mask_i, np.floor(x_i), x_i)
+        # For x_j
+        if hasattr(x_j, "__len__"):
+            mask_j = np.array([var_type[j] != "num"] * len(x_j))
+            x_j = np.where(mask_j, np.floor(x_j), x_j)
 
-    # Create a grid for the two varied dimensions
-    x_i = linspace(X[:, i].min(), X[:, i].max(), num=n_grid)
-    x_j = linspace(X[:, j].min(), X[:, j].max(), num=n_grid)
     X_i, X_j = meshgrid(x_i, x_j)
-
-    # Prepare the grid points for prediction
     grid_points = np.zeros((X_i.size, k))
     grid_points[:, i] = X_i.ravel()
     grid_points[:, j] = X_j.ravel()
@@ -91,65 +115,219 @@ def generate_mesh_grid(X: np.ndarray, i: int, j: int, n_grid: int = 100):
     return X_i, X_j, grid_points
 
 
-def plot_values(
-    ax: "matplotlib.axes.Axes",
+def simple_error_color(z_actual: float, z_predicted: float, eps: float = 1e-3) -> str:
+    """
+    Returns a color string based on the error between actual and predicted values.
+
+    Args:
+        z_actual (float): The actual value.
+        z_predicted (float): The predicted value.
+        eps (float): Tolerance for considering values as close. Default is 1e-3.
+
+    Returns:
+        str: "black" if actual > predicted + eps,
+             "white" if actual < predicted - eps,
+             "grey" otherwise.
+    """
+    # predicted value is smaller than actual value
+    if z_actual > z_predicted + eps:
+        return "black"
+    # predicted value is larger than actual value
+    elif z_actual < z_predicted - eps:
+        return "white"
+    # predicted value is close to actual value
+    else:
+        return "grey"
+
+
+def error_color(z_actual: float, z_predicted: float, eps: float = 1e-4, max_error: float = 1e-3) -> str:
+    """
+    Returns a grayscale color string based on the error between actual and predicted values.
+    Underprediction (z_predicted < z_actual) is black, overprediction is white, zero error is gray.
+    The mapping is linear between -max_error (black) and +max_error (white).
+    """
+    diff = z_predicted - z_actual
+    if abs(diff) <= eps:
+        scale = 0.5  # gray
+    else:
+        # Clamp diff to [-max_error, max_error]
+        diff = max(-max_error, min(diff, max_error))
+        scale = 0.5 + 0.5 * (diff / max_error)
+    scale = min(max(scale, 0.0), 1.0)
+    grey = int(scale * 255)
+    return f"#{grey:02x}{grey:02x}{grey:02x}"
+
+
+def plot_error_points(
+    ax,
     X: np.ndarray,
     y: np.ndarray,
     model,
     i: int,
     j: int,
-    Z,
-    surface_label: str = "Prediction Surface",
-    zlabel: str = "Prediction",
+    eps: float = 1e-4,
+    max_error: float = 1e-3,
     var_names: Optional[List[str]] = None,
-    alpha: float = 0.8,
-    eps: float = 1e-3,
-    cmap: str = "jet",
-    error_surface: bool = False,
+    title: Optional[str] = None,
+    z_mode: str = "actual",  # "actual", "error", or None
 ) -> None:
     """
-    Plot a 3D surface and scatter input points, colored by prediction error.
+    Scatter input points colored by prediction error.
 
     Args:
-        ax (matplotlib.axes.Axes): Matplotlib 3D axis.
+        ax (matplotlib.axes.Axes): The matplotlib axis to plot on.
         X (np.ndarray): Input data, shape (n_samples, k).
         y (np.ndarray): Target values, shape (n_samples,).
         model (object): Fitted model with predict().
         i (int): Index of first varied dimension.
         j (int): Index of second varied dimension.
+        eps (float): Tolerance for coloring points based on prediction error.
+        max_error (float): Maximum error for color scaling.
+        var_names (list of str or None): List of axis labels or None.
+        title (str or None): Title for the plot.
+        z_mode (str): "actual" for z_actual (for 3D), "error" for abs error (for 3D error surface), or None (for 2D).
+    """
+    n, k = X.shape
+    check_ij(i, j, k)
+    for idx in range(n):
+        x_point = X[idx, i]
+        y_point_ = X[idx, j]
+        z_actual = y[idx]
+        z_predicted = model.predict(X[idx].reshape(1, -1))[0]
+        color = error_color(z_actual, z_predicted, eps, max_error)
+        if z_mode == "actual":
+            ax.scatter(x_point, y_point_, z_actual, color=color, s=50, edgecolor="black")
+        elif z_mode == "error":
+            ax.scatter(x_point, y_point_, abs(z_actual - z_predicted), color=color, s=50, edgecolor="black")
+        else:
+            ax.scatter(x_point, y_point_, color=color, s=50, edgecolor="black")
+    if title is not None:
+        ax.set_title(title)
+    if var_names is not None:
+        ax.set_xlabel(var_names[0])
+        ax.set_ylabel(var_names[1])
+    else:
+        ax.set_xlabel(f"Dimension {i}")
+        ax.set_ylabel(f"Dimension {j}")
+
+
+def plot_3d_surface(
+    Z,
+    i: int,
+    j: int,
+    ax: "matplotlib.axes.Axes",
+    X: np.ndarray = None,
+    y: np.ndarray = None,
+    model=None,
+    surface_label: str = "Prediction Surface",
+    zlabel: str = "Prediction",
+    var_names: Optional[List[str]] = None,
+    alpha: float = 0.8,
+    eps: float = 1e-4,
+    max_error: float = 1e-3,
+    cmap: str = "jet",
+    error_surface: bool = False,
+    add_points: bool = False,
+    vmin: float = None,
+    vmax: float = None,
+) -> None:
+    """
+    Plot a 3D surface and scatter input points, colored by prediction error.
+
+    Args:
         Z (tuple or np.ndarray): Surface values to plot, shape matching meshgrid.
+        i (int): Index of first varied dimension.
+        j (int): Index of second varied dimension.
+        ax (matplotlib.axes.Axes): Matplotlib 3D axis.
+        X (np.ndarray): Input data, shape (n_samples, k).
+        y (np.ndarray): Target values, shape (n_samples,).
+        model (object): Fitted model with predict().
         surface_label (str): Title for the surface.
         zlabel (str): Label for the z-axis.
         var_names (list of str or None): List of axis labels or None.
         alpha (float): Surface transparency.
         eps (float): Tolerance for error coloring.
+        max_error (float): Maximum error for color scaling.
         cmap (str): Colormap for the surface.
         error_surface (bool): If True, scatter z is abs(y_actual - y_predicted).
+        add_points (bool): If True, adds scatter points to the surface plot.
 
     Returns:
         None
     """
-    ax.plot_surface(*Z[:2], Z[2], cmap=cmap, alpha=alpha) if isinstance(Z, tuple) else ax.plot_surface(Z[0], Z[1], Z[2], cmap=cmap, alpha=alpha)
+    ax.plot_surface(*Z[:2], Z[2], cmap=cmap, alpha=alpha, vmin=vmin, vmax=vmax) if isinstance(Z, tuple) else ax.plot_surface(Z[0], Z[1], Z[2], cmap=cmap, alpha=alpha, vmin=vmin, vmax=vmax)
     ax.set_title(surface_label)
     ax.set_xlabel(var_names[0] if var_names else f"Dimension {i}")
     ax.set_ylabel(var_names[1] if var_names else f"Dimension {j}")
     ax.set_zlabel(var_names[2] if var_names else zlabel)
-    for idx in range(X.shape[0]):
-        x_point = X[idx, i]
-        y_point = X[idx, j]
-        z_actual = y[idx]
-        z_predicted = model.predict(X[idx].reshape(1, -1))[0]
-        if error_surface:
-            z_scatter = abs(z_actual - z_predicted)
-        else:
-            z_scatter = z_actual
-        if z_actual > z_predicted + eps:
-            color = "red"
-        elif z_actual < z_predicted - eps:
-            color = "green"
-        else:
-            color = "white"
-        ax.scatter(x_point, y_point, z_scatter, color=color, s=50, edgecolor="black")
+    if add_points and X is not None and y is not None and model is not None:
+        plot_error_points(ax, X, y, model, i, j, eps, max_error, var_names, z_mode="error" if error_surface else "actual")
+
+
+def plot_contour_and_err(
+    X_i: np.ndarray,
+    X_j: np.ndarray,
+    Z: np.ndarray,
+    i: int,
+    j: int,
+    ax: "matplotlib.axes.Axes",
+    X: np.ndarray = None,
+    y: np.ndarray = None,
+    model=None,
+    eps: float = 1e-4,
+    max_error: float = 1e-3,
+    var_names: Optional[List[str]] = None,
+    cmap: str = "jet",
+    levels: int = 30,
+    title: str = "Prediction Contour",
+    add_points: bool = False,
+    vmin: float = None,
+    vmax: float = None,
+) -> None:
+    """
+    Plot a filled contour plot with scatter points colored by prediction error.
+
+    Args:
+        X_i (np.ndarray): Meshgrid for the i-th dimension.
+        X_j (np.ndarray): Meshgrid for the j-th dimension.
+        Z (np.ndarray): Contour values (predicted or error), shape matching meshgrid.
+        i (int): Index of first varied dimension.
+        j (int): Index of second varied dimension.
+        ax (matplotlib.axes.Axes): The matplotlib axis to plot on.
+        X (np.ndarray): Input data, shape (n_samples, k).
+        y (np.ndarray): Target values, shape (n_samples,).
+        model (object): Fitted model with predict().
+        eps (float): Tolerance for coloring points based on prediction error.
+        max_error (float): Maximum error for color scaling.
+        var_names (list of str or None): List of axis labels or None.
+        cmap (str): Colormap for the contour plot.
+        levels (int): Number of contour levels.
+        title (str): Title for the plot.
+        add_points (bool): If True, adds scatter points to the contour plot.
+        vmin (float): Minimum value for color scaling.
+        vmax (float): Maximum value for color scaling.
+
+    Returns:
+        None
+    """
+    contour = ax.contourf(X_i, X_j, Z, cmap=cmap, levels=levels, vmin=vmin, vmax=vmax)
+    plt.colorbar(contour, ax=ax)
+    if add_points and X is not None and y is not None and model is not None:
+        plot_error_points(ax, X, y, model, i, j, eps, max_error, var_names, title, z_mode=None)
+
+
+def check_ij(i: int, j: int, k: int) -> None:
+    """
+    Check if indices i and j are valid for the number of features k.
+    Args:
+        i (int): Index of the first dimension.
+        j (int): Index of the second dimension.
+        k (int): Total number of features.
+    """
+    if i >= k or j >= k:
+        raise ValueError(f"Dimensions i and j must be less than the number of features (k={k}).")
+    if i == j:
+        raise ValueError("Dimensions i and j must be different.")
 
 
 def plotkd(
@@ -159,11 +337,16 @@ def plotkd(
     i: int = 0,
     j: int = 1,
     show: Optional[bool] = True,
-    alpha=0.8,
-    eps=1e-3,
+    alpha: float = 0.8,
+    eps: float = 1e-4,
+    max_error: float = 1e-3,
     var_names: Optional[List[str]] = None,
-    cmap="jet",
-    n_grid: int = 100,
+    var_type: Optional[List[str]] = None,
+    cmap: str = "jet",
+    num: int = 100,
+    vmin: Optional[float] = None,
+    vmax: Optional[float] = None,
+    add_points: bool = False,
 ) -> None:
     """
     Plots the Kriging surrogate model for k-dimensional input data by varying two dimensions (i, j).
@@ -176,18 +359,32 @@ def plotkd(
         j (int): Index of the second dimension to vary. Default is 1.
         show (bool): If True, displays the plot. Default is True.
         alpha (float): Transparency of the surface plot. Default is 0.8.
-        eps (float): Tolerance for coloring points based on prediction error. Default is 1e-3.
+        eps (float): Tolerance for coloring points based on prediction error. Default is 1e-4.
+        max_error (float): Maximum error for color scaling. Default is 1e-3.
         var_names (list of str, optional): List of variable names for axis labeling. If None, generic labels are used.
+        var_type (list of str, optional): List of variable types for each dimension. Can be either "num", "int", or "factor".
         cmap (str): Colormap for the surface and contour plots. Default is "jet".
-        n_grid (int): Number of grid points per dimension for the mesh grid. Default is 100.
+        num (int): Number of grid points per dimension for the mesh grid. Default is 100.
+        vmin (float, optional): Minimum value for the color scale. If None, determined from predictions.
+        vmax (float, optional): Maximum value for the color scale. If None, determined from predictions.
+        add_points (bool): If True, adds scatter points to the surface and contour plots. Default is False.
+
+    Examples:
+        >>> import numpy as np
+        >>> from spotpython.surrogate.kriging import Kriging
+        >>> from spotpython.surrogate.plot import plotkd
+        >>> # Training data
+        >>> X_train = np.random.rand(100, 3)  # 100 samples with 3 dimensions
+        >>> y_train = np.sin(X_train[:, 0]) + np.cos(X_train[:, 1]) + X_train[:, 2]  # Example target function
+        >>> # Initialize and fit the Kriging model
+        >>> model = Kriging().fit(X_train, y_train)
+        >>> # Plot the Kriging surrogate for dimensions 0 and 1
+        >>> plotkd(model, X_train, y_train, i=0, j=1, show=True)
+
     """
     k = X.shape[1]
-    if i >= k or j >= k:
-        raise ValueError(f"Dimensions i and j must be less than the number of features (k={k}).")
-    if i == j:
-        raise ValueError("Dimensions i and j must be different.")
-
-    X_i, X_j, grid_points = generate_mesh_grid(X, i, j, n_grid)
+    check_ij(i, j, k)
+    X_i, X_j, grid_points = generate_mesh_grid(X, i, j, num, var_type=var_type)
 
     # Predict the values and standard deviations
     y_pred, y_std = model.predict(grid_points, return_std=True)
@@ -198,81 +395,95 @@ def plotkd(
 
     # Plot predicted values
     ax1 = fig.add_subplot(221, projection="3d")
-    plot_values(
+    plot_3d_surface(
+        (X_i, X_j, Z_pred),
+        i,
+        j,
         ax1,
         X,
         y,
         model,
-        i,
-        j,
-        (X_i, X_j, Z_pred),
         surface_label="Prediction Surface",
         zlabel="Prediction",
         var_names=var_names,
         alpha=alpha,
         eps=eps,
+        max_error=max_error,
         cmap=cmap,
         error_surface=False,
+        vmin=vmin,
+        vmax=vmax,
+        add_points=add_points,
     )
 
     # Plot prediction error
     ax2 = fig.add_subplot(222, projection="3d")
-    plot_values(
+    plot_3d_surface(
+        (X_i, X_j, Z_std),
+        i,
+        j,
         ax2,
         X,
         y,
         model,
-        i,
-        j,
-        (X_i, X_j, Z_std),
         surface_label="Prediction Error Surface",
         zlabel="Error",
         var_names=var_names,
         alpha=alpha,
         eps=eps,
+        max_error=max_error,
         cmap=cmap,
         error_surface=True,
+        vmin=vmin,
+        vmax=vmax,
+        add_points=add_points,
     )
 
     # Contour plot of predicted values
     ax3 = fig.add_subplot(223)
-    contour = ax3.contourf(X_i, X_j, Z_pred, cmap=cmap, levels=30)
-    plt.colorbar(contour, ax=ax3)
-    for idx in range(X.shape[0]):
-        x_point = X[idx, i]
-        y_point = X[idx, j]
-        z_actual = y[idx]
-        z_predicted = model.predict(X[idx].reshape(1, -1))[0]
-        if z_actual > z_predicted + eps:
-            color = "red"
-        elif z_actual < z_predicted - eps:
-            color = "green"
-        else:
-            color = "white"
-        ax3.scatter(x_point, y_point, color=color, s=50, edgecolor="black")
-    ax3.set_title("Prediction Contour")
-    ax3.set_xlabel(var_names[0] if var_names else f"Dimension {i}")
-    ax3.set_ylabel(var_names[1] if var_names else f"Dimension {j}")
+    plot_contour_and_err(
+        X_i,
+        X_j,
+        Z_pred,
+        i,
+        j,
+        ax3,
+        X,
+        y,
+        model,
+        eps=eps,
+        max_error=max_error,
+        var_names=var_names,
+        cmap=cmap,
+        levels=30,
+        title="Prediction Contour",
+        vmin=vmin,
+        vmax=vmax,
+        add_points=add_points,
+    )
 
     # Contour plot of prediction error
     ax4 = fig.add_subplot(224)
-    contour = ax4.contourf(X_i, X_j, Z_std, cmap=cmap, levels=30)
-    plt.colorbar(contour, ax=ax4)
-    for idx in range(X.shape[0]):
-        x_point = X[idx, i]
-        y_point = X[idx, j]
-        z_actual = y[idx]
-        z_predicted = model.predict(X[idx].reshape(1, -1))[0]
-        if z_actual > z_predicted + eps:
-            color = "red"
-        elif z_actual < z_predicted - eps:
-            color = "green"
-        else:
-            color = "white"
-        ax4.scatter(x_point, y_point, color=color, s=50, edgecolor="black")
-    ax4.set_title("Error Contour")
-    ax4.set_xlabel(var_names[0] if var_names else f"Dimension {i}")
-    ax4.set_ylabel(var_names[1] if var_names else f"Dimension {j}")
+    plot_contour_and_err(
+        X_i,
+        X_j,
+        Z_std,
+        i,
+        j,
+        ax4,
+        X,
+        y,
+        model,
+        eps=eps,
+        max_error=max_error,
+        var_names=var_names,
+        cmap=cmap,
+        levels=30,
+        title="Error Contour",
+        vmin=vmin,
+        vmax=vmax,
+        add_points=add_points,
+    )
 
     if show:
         plt.show()
@@ -306,7 +517,7 @@ def plot_3d_contour(X, Y, Z, vmin, vmax, var_name=None, i=0, j=1, show=True, fil
     Examples:
         >>> # Example 1: Using output from Spot
         >>> # Assume S is a Spot object with a fitted surrogate
-        >>> plot_data = S.prepare_plot(i=0, j=1, n_grid=100)
+        >>> plot_data = S.prepare_plot(i=0, j=1, num=100)
         >>> from spotpython.surrogate.plot import plot_3d_contour
         >>> plot_3d_contour(
         ...     plot_data,
